# -*- coding:Utf-8 -*-
# ../addons/eventscripts/_libs/python/insertcmdlib.py
# allow you to execute a server command. Can execute a function when command has been passed

import es, psyco
psyco.full()



esInfo = es.AddonInfo()
esInfo.author = "DoCky"
esInfo.version = "1.001p"
esInfo.name = "InsertCmdLib"
esInfo.basename = "insertcmd"
esInfo.url = "http://addons.eventscripts.com/addons/view/insertcmdlib"
esInfo.description = "allow you to execute a server command. Can execute a ESP function when command has been passed"

version = es.ServerVar("insertcmdlib", "", " Created by DoCky -")
version.set(esInfo.version)
version.makepublic()



_insertcmd = es.server.insertcmd

class InsertQueue(list):
    """ InsertQueue object """

    def _insertcmd(self):
        try:
            # get instance of current running command
            instance = self[0]
            # set status of execution
            instance.executed = True

            # remove command from queue (insertqueue list)
            self.remove(instance)
            # execute callback function with InsertCmd object as parameter
            if callable(instance.function): instance.function(instance)
            # if commands remains in list, execute them...
            if self: self[0]._prompt()
        except IndexError: es.dbgmsg(1, "insertcmdlib: command has been canceled or script is no longer active")

    def getInsertCmd(self, name):
        for instance in self:
            if instance.name == name: return instance

# store InsertQueue object
_insertqueue = InsertQueue()
# register funbctions for internal command
es.addons.registerBlock("_insertcmdlib", "_insertcmd", _insertqueue._insertcmd)
# register server commands
es.regcmd("_icl-cmd_", "_insertcmdlib/_insertcmd", "insertcmdlib: internal command")



# create an object for each inserted command
class InsertCmd(object):
    """ InsertCmd object """

    def __init__(self, command = "", function = None, parameters = (), name = ""):
        # store command
        self.command = str(command) if command else ""
        # store function to call back
        self.function = function if function else None
        # store parameters
        self.parameters = parameters if parameters else ()
        # store name of thread (like a targetname)
        self.name = str(name) if name else ""

        # set status of command (True if it was executed, else False)
        self.executed = False

    def __getitem__(self, name):
        return getattr(self, name)

    def _prompt(self):
        # execute command...
        _insertcmd(self.command + ";_icl-cmd_")

    def execute(self):
        """ execute command """
        # set status of execution
        self.executed = False
        # insert command into the queue
        _insertqueue.append(self)
        # try to execute command...
        if len(_insertqueue) == 1: self._prompt()

    def cancel(self):
        """ cancel registered command """
        if self in _insertqueue:
            _insertqueue.remove(self)
            return True
        return False



# execute command...
def execute(command = "", function = None, parameters = (), name = ""):
    InsertCmd(command, function, parameters, name).execute()

# cancel command
def cancel(name):
    instance = getInsertCmd(name)
    if instance is None: return False
    while instance:
        _insertqueue.remove(instance)
        instance = getInsertCmd(name)
    return True

# get instance
def getInsertCmd(name):
    return _insertqueue.getInsertCmd(name)
